@@ -9,5 +9,13 @@ ERROR_CODES = {
     "-40105": "Method does not exist",
     "-40101": "Parameter to set does not exist",
     "-40209": "Invalid login credentials",
+    "-64304": "Maximum Pan/Tilt range reached",
+    "-71103": "User ID is not authorized",
+    "72102": "Quick response id not found",
 }
 MAX_LOGIN_RETRIES = 2
+
+
+class EncryptionMethod:
+    MD5 = "md5"
+    SHA256 = "sha256"
