@@ -1,17 +1,16 @@
-from pytapo.media_stream.convert import Convert
+from .convert import Convert
 from pytapo import Tapo
 from datetime import datetime
+from json import JSONDecodeError
+from ._utils import StreamType
 
 import json
 import os
 import hashlib
+import aiofiles
 
 
 class Downloader:
-    scriptStartTime = int(
-        datetime.now().timestamp()
-    )  # keeps track of when was class first imported in GMT
-
     FRESH_RECORDING_TIME_SECONDS = 60
 
     def __init__(
@@ -19,6 +18,7 @@ class Downloader:
         tapo: Tapo,
         startTime: int,
         endTime: int,
+        timeCorrection: int,
         outputDirectory="./",
         padding=None,
         overwriteFiles=None,
@@ -30,6 +30,7 @@ class Downloader:
         self.endTime = endTime
         self.padding = padding
         self.fileName = fileName
+        self.timeCorrection = timeCorrection
         if padding is None:
             self.padding = 5
         else:
@@ -42,13 +43,11 @@ class Downloader:
         else:
             self.window_size = int(window_size)
 
-    def md5(self, fileName):
+    async def md5(self, fileName):
         if os.path.isfile(fileName):
-            with open(fileName, "rb") as f:
-                file_hash = hashlib.md5()
-                while chunk := f.read(8192):
-                    file_hash.update(chunk)
-            return file_hash.hexdigest()
+            async with aiofiles.open(fileName, "rb") as file:
+                contents = await file.read()
+            return hashlib.md5(contents).hexdigest()
         return False
 
     async def downloadFile(self, callbackFunc=None):
@@ -61,7 +60,7 @@ class Downloader:
         if callbackFunc is not None:
             callbackFunc("Finished download")
 
-        md5Hash = self.md5(status["fileName"])
+        md5Hash = await self.md5(status["fileName"])
 
         status["md5"] = "" if md5Hash is False else md5Hash
 
@@ -84,7 +83,12 @@ class Downloader:
                 )
             else:
                 fileName = self.outputDirectory + self.fileName
-            if self.scriptStartTime - self.FRESH_RECORDING_TIME_SECONDS < self.endTime:
+            if (
+                datetime.now().timestamp()
+                - self.FRESH_RECORDING_TIME_SECONDS
+                - self.timeCorrection
+                < self.endTime
+            ):
                 currentAction = "Recording in progress"
                 yield {
                     "currentAction": currentAction,
@@ -104,7 +108,7 @@ class Downloader:
                 downloading = False
             else:
                 convert = Convert()
-                mediaSession = self.tapo.getMediaSession()
+                mediaSession = self.tapo.getMediaSession(StreamType.Download)
                 if retry:
                     mediaSession.set_window_size(50)
                 else:
@@ -166,9 +170,39 @@ class Downloader:
                                     "progress": 0,
                                     "total": 0,
                                 }
-                                convert.save(fileName, segmentLength)
+                                await convert.save(fileName, segmentLength)
                                 downloading = False
                                 break
+                        # in case a finished stream notification is caught, save the chunks as is
+                        elif resp.mimetype == "application/json":
+                            try:
+                                json_data = json.loads(resp.plaintext.decode())
+
+                                if (
+                                    "type" in json_data
+                                    and json_data["type"] == "notification"
+                                    and "params" in json_data
+                                    and "event_type" in json_data["params"]
+                                    and json_data["params"]["event_type"]
+                                    == "stream_status"
+                                    and "status" in json_data["params"]
+                                    and json_data["params"]["status"] == "finished"
+                                ):
+                                    downloadedFull = True
+                                    currentAction = "Converting"
+                                    yield {
+                                        "currentAction": currentAction,
+                                        "fileName": fileName,
+                                        "progress": 0,
+                                        "total": 0,
+                                    }
+                                    await convert.save(fileName, convert.getLength())
+                                    downloading = False
+                                    break
+                            except JSONDecodeError:
+                                self.tapo.debugLog(
+                                    "Unable to parse JSON sent from device"
+                                )
                     if downloading:
                         # Handle case where camera randomly stopped respoding
                         if not downloadedFull and not retry:
@@ -193,7 +227,7 @@ class Downloader:
                                     "progress": 0,
                                     "total": 0,
                                 }
-                                convert.save(fileName, segmentLength)
+                                await convert.save(fileName, segmentLength)
                             else:
                                 currentAction = "Giving up"
                                 yield {
