@@ -4,25 +4,27 @@ import json
 import logging
 import random
 import warnings
+import urllib.parse
+from ..const import EncryptionMethod
 from asyncio import StreamReader, StreamWriter, Task, Queue
 from json import JSONDecodeError
 from typing import Optional, Mapping, Generator, MutableMapping
 
 from rtp import PayloadType
 
-from pytapo.media_stream._utils import (
+from ._utils import (
     generate_nonce,
-    md5digest,
+    pwd_digest,
     parse_http_response,
     parse_http_headers,
 )
-from pytapo.media_stream.crypto import AESHelper
-from pytapo.media_stream.error import (
+from .crypto import AESHelper
+from .error import (
     HttpStatusCodeException,
     KeyExchangeMissingException,
 )
-from pytapo.media_stream.response import HttpMediaResponse
-from pytapo.media_stream.tsReader import TSReader
+from .response import HttpMediaResponse
+from .tsReader import TSReader
 
 logger = logging.getLogger(__name__)
 
@@ -33,16 +35,21 @@ class HttpMediaSession:
         ip: str,
         cloud_password: str,
         super_secret_key: str,
+        encryptionMethod: EncryptionMethod,
         window_size=500,  # 500 is a sweet point for download speed
         port: int = 8800,
         username: str = "admin",
         multipart_boundary: bytes = b"--client-stream-boundary--",
+        query_params: dict = {},
     ):
         self.ip = ip
         self.window_size = window_size
         self.cloud_password = cloud_password
         self.super_secret_key = super_secret_key
-        self.hashed_password = md5digest(cloud_password.encode()).decode()
+        self.encryptionMethod = encryptionMethod
+        self.hashed_password = pwd_digest(
+            cloud_password.encode(), self.encryptionMethod
+        ).decode()
         self.port = port
         self.username = username
         self.client_boundary = multipart_boundary
@@ -62,6 +69,10 @@ class HttpMediaSession:
 
         self._sequence_numbers: MutableMapping[int, Queue] = {}
         self._sessions: MutableMapping[int, Queue] = {}
+        self.query_params = query_params
+        self.query_params_str = ""
+        if any(query_params):
+            self.query_params_str = f"?{urllib.parse.urlencode(query_params)}"
 
     def set_window_size(self, window_size):
         self.window_size = window_size
@@ -75,7 +86,7 @@ class HttpMediaSession:
         return self
 
     async def start(self):
-        req_line = b"POST /stream HTTP/1.1"
+        req_line = f"POST /stream{self.query_params_str} HTTP/1.1".encode()
         headers = {
             b"Content-Type": "multipart/mixed;boundary={}".format(
                 self.client_boundary.decode()
@@ -83,11 +94,13 @@ class HttpMediaSession:
             b"Connection": b"keep-alive",
             b"Content-Length": b"-1",
         }
+        if self.query_params_str:
+            headers[b"X-Client-UUID"] = self.query_params["playerId"].encode()
         try:
             self._reader, self._writer = await asyncio.open_connection(
                 self.ip, self.port
             )
-            logger.info("Connected to the media streaming server")
+            logger.debug("Connected to the media streaming server")
 
             # Step one: perform unauthenticated request
             await self._send_http_request(req_line, headers)
@@ -151,7 +164,10 @@ class HttpMediaSession:
             # Ensure the request was successful
             data = await self._reader.readuntil(b"\r\n\r\n")
             res_line, headers_block = data.split(b"\r\n", 1)
+            logger.debug("Before parsing http response")
+            logger.debug(res_line)
             _, status_code, _ = parse_http_response(res_line)
+            logger.debug("After parsing http response")
             if status_code != 200:
                 raise HttpStatusCodeException(status_code)
 
@@ -185,6 +201,7 @@ class HttpMediaSession:
                 self._key_exchange.encode(),
                 self.cloud_password.encode(),
                 self.super_secret_key.encode(),
+                self.encryptionMethod,
             )
 
             logger.debug("AES key exchange performed")
@@ -230,13 +247,9 @@ class HttpMediaSession:
 
             logger.debug("Handling new server response")
 
-            # print("got response")
-
             # Read and parse headers
             headers_block = await self._reader.readuntil(b"\r\n\r\n")
             headers = parse_http_headers(headers_block)
-            # print(headers)
-
             mimetype = headers["Content-Type"]
             length = int(headers["Content-Length"])
             encrypted = bool(int(headers["X-If-Encrypt"]))
@@ -246,26 +259,14 @@ class HttpMediaSession:
             if "X-Data-Sequence" in headers:
                 seq = int(headers["X-Data-Sequence"])
 
-            # print(headers)
-
             # Now we know the content length, let's read it and decrypt it
             json_data = None
-            # print("TEST0")
             data = await self._reader.readexactly(length)
             if encrypted:
-                # print("encrypted")
                 ciphertext = data
-                # print("TEST1")
                 try:
-                    # print("lolo")
-                    # print(ciphertext)
                     plaintext = self._aes.decrypt(ciphertext)
-                    # if length == 384:
-                    # print(plaintext)
-                    # print("lala")
-                    # print(plaintext)
                 except ValueError as e:
-                    # print(e)
                     if "padding is incorrect" in e.args[0].lower():
                         e = ValueError(
                             e.args[0]
@@ -276,26 +277,38 @@ class HttpMediaSession:
                 except Exception as e:
                     plaintext = e
             else:
-                # print("plaintext")
                 ciphertext = None
                 plaintext = data
-            # print(plaintext)
+
+            queue: Optional[Queue] = None
+
             # JSON responses sometimes have the above info in the payload,
             # not the headers. Let's parse it.
             if mimetype == "application/json":
                 try:
                     json_data = json.loads(plaintext.decode())
                     if "seq" in json_data:
-                        # print("Setting seq")
                         seq = json_data["seq"]
                     if "params" in json_data and "session_id" in json_data["params"]:
                         session = int(json_data["params"]["session_id"])
-                        # print("Setting session")
+                    elif (
+                        "type" in json_data
+                        and json_data["type"] == "notification"
+                        and "params" in json_data
+                        and "event_type" in json_data["params"]
+                        and json_data["params"]["event_type"] == "stream_status"
+                        and "status" in json_data["params"]
+                        and json_data["params"]["status"] == "finished"
+                        and len(self._sessions) > 0
+                    ):
+                        # use next queue item to inject this info, since no id session can be inferred
+                        queue = next(iter(self._sessions.values()))
                 except JSONDecodeError:
                     logger.warning("Unable to parse JSON sent from device")
 
             if (
-                (session is None)
+                (queue is None)
+                and (session is None)
                 and (seq is None)
                 or (
                     (session is not None)
@@ -310,15 +323,10 @@ class HttpMediaSession:
                 )
                 continue
 
-            # # Update our own sequence numbers to avoid collisions
-            # if (seq is not None) and (seq > self._seq_counter):
-            #     self._seq_counter = seq + 1
-
-            queue: Optional[Queue] = None
-
             # Move queue to use sessions from now on
             if (
-                (session is not None)
+                (queue is None)
+                and (session is not None)
                 and (seq is not None)
                 and (session not in self._sessions)
                 and (seq in self._sequence_numbers)
@@ -344,7 +352,6 @@ class HttpMediaSession:
             )
 
             if seq is not None and seq % self.window_size == 0:  # never ack live stream
-                # print("sending ack")
                 data = {
                     "type": "notification",
                     "params": {"event_type": "stream_sequence"},
@@ -361,7 +368,6 @@ class HttpMediaSession:
                 await self._send_http_request(b"--" + self.client_boundary, headers)
                 chunk_size = 4096
                 for i in range(0, len(data), chunk_size):
-                    # print(data[i : i + chunk_size])
                     self._writer.write(data[i : i + chunk_size])
                     await self._writer.drain()
 
@@ -449,9 +455,8 @@ class HttpMediaSession:
         await self._send_http_request(b"--" + self.client_boundary, headers)
 
         chunk_size = 4096
-        # print("Sending:")
+
         for i in range(0, len(data), chunk_size):
-            # print(data[i : i + chunk_size])
             self._writer.write(data[i : i + chunk_size])
             await self._writer.drain()
 
@@ -502,7 +507,7 @@ class HttpMediaSession:
                     session = resp.session
                 if resp.encrypted and isinstance(resp.plaintext, Exception):
                     raise resp.plaintext
-                # print(resp.plaintext)
+
                 tsReader.setBuffer(list(resp.plaintext))
                 pkt = tsReader.getPacket()
                 if pkt:
